<?php

namespace Drupal\orejime\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form controller for Orejime edit forms.
 *
 * @ingroup orejime
 */
class OrejimeForm extends ContentEntityForm {

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    /* @var $entity \Drupal\orejime\Entity\Orejime */
    $form = parent::buildForm($form, $form_state);
    if (!$this->entity->isNew()) {
      $form['new_revision'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Create new revision'),
        '#default_value' => FALSE,
        '#weight' => 20,
      ];
    }

    $entity = $this->entity;

    return $form;
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    if ($this->nameAlreadyUsed($form_state->getValue('name')[0]['value'], $this->entity->id())) {
      $form_state->setErrorByName('name', t('System Name must be unique.'));
    }
    return parent::validateForm($form, $form_state);
  }

  /**
   * If a name is already used
   * --------------
   * @param $name
   * @param null $id
   * @return array|int
   */
  public function nameAlreadyUsed($name, $id = NULL) {
    $query = \Drupal::entityQuery('orejime_service');
    if ($id) {
      $query->condition('oid', $id, '!=');
    }
    $query->condition('name', $name, '=');
    $result = $query->execute();
    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $entity = $this->entity;

    // Save as a new revision if requested to do so.
    if (!$form_state->isValueEmpty('new_revision') && $form_state->getValue('new_revision') != FALSE) {
      $entity->setNewRevision();

      // If a new revision is created, save the current user as revision author.
      $entity->setRevisionCreationTime(REQUEST_TIME);
      $entity->setRevisionUserId(\Drupal::currentUser()->id());
    }
    else {
      $entity->setNewRevision(FALSE);
    }

    $status = parent::save($form, $form_state);

    switch ($status) {
      case SAVED_NEW:
        drupal_set_message($this->t('Created the %label Orejime.', [
          '%label' => $entity->label(),
        ]));
        break;

      default:
        drupal_set_message($this->t('Saved the %label Orejime.', [
          '%label' => $entity->label(),
        ]));
    }
    $form_state->setRedirect('entity.orejime_service.collection');
  }

}
