<?php

namespace Drupal\orejime\Services;

use Drupal\orejime\Entity\Orejime;

class OrejimeManager {

  /**
   * Get services Orejime for Cookie Manage
   * --------------
   * @return array
   */
  public function getServicesManage() {
    $return = &drupal_static('get_orejime_services_manage');
    if (!$return) {
      $return = ['services' => [], 'purposes' => []];
      $purposes = [];
      $services = Orejime::loadMultiple($this->getAllServices());
      foreach ($services as $service) {
        $purpose = $service->getPurposesArray();
        $return['services'][$service->label()] = [
          'id' => $service->oid,
          'label' => $service->getLabel(),
          'description' => $service->getDescription(),
          'required' => $service->getRequired(),
          'cookies' => $service->getCookiesArray(),
          'scripts' => $service->getScriptsArray(),
          'default' => $service->getDefault(),
          'purposes' => $purpose
        ];
        $purposes = array_merge($purposes, $purpose);
      }
      $return['purposes'] = array_unique($purposes);
    }
    return $return;
  }

  /**
   * Get all services Orejime
   * --------------
   * @return array
   */
  public function getAllServices() {
    $query = \Drupal::entityQuery('orejime_service');
    $query->condition('status', TRUE, '=');
    return $query->execute();
  }

  /**
   * Get scripts
   * --------------
   * @return array|mixed
   */
  public function getAllScripts() {
    module_load_include('inc', 'orejime', 'orejime.pages');
    $scripts = [];
    $services = $this->getServicesManage();
    foreach ($services['services'] as $id => $service) {
      foreach ($service['scripts'] as $script_service) {
        $scripts[] = array('type' => $id, 'script' => $script_service);
      }
    }
    return $scripts;
  }

  /**
   * Set opt-in to js
   * --------------
   * @param $javascript
   */
  public function setOptIn(&$javascript){
    $scripts = $this->getAllScripts();
    $keys = array_keys($javascript);
    foreach ($scripts as $script){
      $value = preg_grep('/' . preg_quote($script['script'] . '/') ,  $keys);
      if(count($value) > 0){
        $value = reset($value);
        $javascript[$value]['preprocess'] = FALSE;
        $javascript[$value]['attributes'] = ['data-src' => $value, 'type' => 'opt-in', 'data-name' => $script['type'], 'data-type' => 'application/javascript'];
      }
    }
  }
}
